@@ -1,6 +1,7 @@
 #!/usr/bin/env python3
 
 import logging
+import os
 import re
 from dataclasses import dataclass
 from typing import List, NamedTuple, Optional, Set, Tuple
@@ -9,6 +10,7 @@ import ghstack
 import ghstack.git
 import ghstack.github
 import ghstack.github_utils
+import ghstack.gpg_sign
 import ghstack.logs
 import ghstack.shell
 from ghstack.types import (GhNumber, GitCommitHash, GitHubNumber,
@@ -90,7 +92,15 @@ def branch_orig(username: str, ghnum: GhNumber) -> GitCommitHash:
     return branch(username, ghnum, "orig")
 
 
-STACK_HEADER = "Stack from [ghstack](https://{github_url}/ezyang/ghstack)"
+RE_MENTION = re.compile(r'@([a-z\d](?:[a-z\d]|-(?=[a-z\d])){0,38})', re.I)
+
+
+# Replace GitHub mentions with non mentions, to prevent spamming people
+def strip_mentions(body: str) -> str:
+    return RE_MENTION.sub(r'\1', body)
+
+
+STACK_HEADER = "Stack from [ghstack](https://{github_url}/ezyang/ghstack) (oldest at bottom)"
 
 
 @dataclass
@@ -152,7 +162,7 @@ def main(*,
     repo_id = repo["id"]
     default_branch = repo["default_branch"]
 
-    sh.git("fetch", remote_name)
+    sh.git("fetch", "--prune", remote_name)
     base = GitCommitHash(sh.git("merge-base", f"{remote_name}/{default_branch}", "HEAD"))
 
     # compute the stack of commits to process (reverse chronological order),
@@ -166,6 +176,8 @@ def main(*,
 
     assert len(stack) > 0
 
+    run_pre_ghstack_hook(sh, base, stack[0].oid)
+
     ghstack.logs.record_status(
         "{} \"{}\"".format(stack[0].oid[:9], stack[0].title))
 
@@ -587,8 +599,8 @@ Since we cannot proceed, ghstack will abort now.
         self.seen_ghnums.add(ghnum)
 
         new_pull = GitCommitHash(
-            self.sh.git("commit-tree", tree,
-                        "-p", self.base_commit,
+            self.sh.git("commit-tree", *ghstack.gpg_sign.gpg_args_if_necessary(self.sh),
+                        "-p", self.base_commit, tree,
                         input=commit.summary + "\n\n[ghstack-poisoned]"))
 
         # Push the branches, so that we can create a PR for them
@@ -627,7 +639,7 @@ Since we cannot proceed, ghstack will abort now.
                       "ghstack-source-id: {sourceid}\n"
                       "Pull Request resolved: "
                       "https://{github_url}/{owner}/{repo}/pull/{number}"
-                      .format(commit_msg=commit.summary.rstrip(),
+                      .format(commit_msg=strip_mentions(commit.summary.rstrip()),
                               owner=self.repo_owner,
                               repo=self.repo_name,
                               number=number,
@@ -639,8 +651,9 @@ Since we cannot proceed, ghstack will abort now.
         # --amend does...)
         new_orig = GitCommitHash(self.sh.git(
             "commit-tree",
-            tree,
+            *ghstack.gpg_sign.gpg_args_if_necessary(self.sh),
             "-p", self.base_orig,
+            tree,
             input=commit_msg))
 
         self.stack_meta.append(DiffMeta(
@@ -774,7 +787,7 @@ Since we cannot proceed, ghstack will abort now.
         # commit message is not supposed to "matter" anymore.  orig
         # still uses the original commit message, however, because
         # it's supposed to be the "original".
-        non_orig_commit_msg = RE_STACK.sub('', elab_commit.body)
+        non_orig_commit_msg = strip_mentions(RE_STACK.sub('', elab_commit.body))
 
         if orig_base_hash == self.base_commit:
 
@@ -799,10 +812,11 @@ Since we cannot proceed, ghstack will abort now.
             # the old base as an ancestor of our commit, but
             # it's better than nothing.
             new_base = GitCommitHash(self.sh.git(
-                "commit-tree", self.base_tree,
+                "commit-tree", *ghstack.gpg_sign.gpg_args_if_necessary(self.sh),
                 "-p",
                 self.remote_name + "/" + branch_base(username, ghnum),
                 *(() if same_stack_base else ("-p", self.stack_base)),
+                self.base_tree,
                 input='Update base for {} on "{}"\n\n{}\n\n[ghstack-poisoned]'
                       .format(self.msg, elab_commit.title,
                               non_orig_commit_msg)))
@@ -822,9 +836,10 @@ Since we cannot proceed, ghstack will abort now.
             return
 
         new_pull = GitCommitHash(self.sh.git(
-            "commit-tree", tree,
+            "commit-tree", *ghstack.gpg_sign.gpg_args_if_necessary(self.sh),
             "-p", self.remote_name + "/" + branch_head(username, ghnum),
             *base_args,
+            tree,
             input='{} on "{}"\n\n{}\n\n[ghstack-poisoned]'.format(self.msg, elab_commit.title, non_orig_commit_msg)))
 
         # Perform what is effectively an interactive rebase
@@ -838,8 +853,8 @@ Since we cannot proceed, ghstack will abort now.
 
         logging.info("Restacking commit on {}".format(self.base_orig))
         new_orig = GitCommitHash(self.sh.git(
-            "commit-tree", tree,
-            "-p", self.base_orig, input=summary))
+            "commit-tree", *ghstack.gpg_sign.gpg_args_if_necessary(self.sh),
+            "-p", self.base_orig, tree, input=summary))
 
         push_branches = (
             (new_base, "base"),
@@ -1039,3 +1054,15 @@ Since we cannot proceed, ghstack will abort now.
             if noop_pr:
                 print()
                 print("I did NOT close or update PRs previously associated with these commits.")
+
+
+def run_pre_ghstack_hook(sh: ghstack.shell.Shell, base_commit: str, top_commit: str) -> None:
+    """If a `pre-ghstack` git hook is configured, run it."""
+    default_hooks_path = os.path.join(sh.git("rev-parse", "--show-toplevel"), ".git/hooks")
+    hooks_path = sh.git("config", "--default", default_hooks_path, "--get", "core.hooksPath")
+    hook_file = os.path.join(hooks_path, "pre-ghstack")
+
+    if not os.path.isfile(hook_file) or not os.access(hook_file, os.X_OK):
+        return
+
+    sh.sh(hook_file, base_commit, top_commit, stdout=None)
